.\" Automatically generated by Pandoc 2.5
.\"
.TH "dgw::tvmixins 0.3" "" "" "" ""
.hy
.SS NAME
.PP
\f[B]dgw::tvmixins\f[R] \- implementations of various extensions for the
\f[I]ttk::treeview\f[R] widget which can be added dynamically using
chaining of commands at widget creation or using the
\f[I]dgw::mixin\f[R] command after widget creation.
.SS TABLE OF CONTENTS
.IP \[bu] 2
SYNOPSIS
.IP \[bu] 2
DESCRIPTION
.IP \[bu] 2
WIDGET COMMANDS
.RS 2
.IP \[bu] 2
dgw::mixin
.IP \[bu] 2
dgw::tvband
.IP \[bu] 2
dgw::tvedit
.IP \[bu] 2
dgw::tvfilebrowser
.IP \[bu] 2
dgw::tvksearch
.IP \[bu] 2
dgw::tvsortable
.IP \[bu] 2
dgw::tvtooltip
.IP \[bu] 2
dgw::tvtree
.RE
.IP \[bu] 2
EXAMPLE
.IP \[bu] 2
INSTALLATION
.IP \[bu] 2
DEMO
.IP \[bu] 2
DOCUMENTATION
.IP \[bu] 2
SEE ALSO
.IP \[bu] 2
CHANGES
.IP \[bu] 2
TODO
.IP \[bu] 2
AUTHORS
.IP \[bu] 2
COPYRIGHT
.IP \[bu] 2
LICENSE
.SS SYNOPSIS
.IP
.nf
\f[C]
package require Tk
package require snit
package require dgw::tvmixins
::dgw::mixin pathName widgetAdaptor ?options?
::dgw::tvband [ttk::treeview pathName ?options?] ?options?
::dgw::tvedit [ttk::treeview pathName ?options?] ?options?
::dgw::tvfilebrowser [ttk::treeview pathName ?options?] ?options?
::dgw::tvksearch [ttk::treeview pathName ?options?] ?options?
::dgw::tvsortable [ttk::treeview pathName ?options?] ?options?
::dgw::tvtooltip [ttk::treeview pathName ?options?] ?options?
::dgw::tvtree [ttk::treeview pathName ?options?] ?options?
::dgw::tvfilebrowser [dgw::tvband [dgw::tvsortable [dgw::tvsearch \[rs]
     [ttk::treeview pathName ?options?] ?options?] ?options?] ?options?]
set tv [ttk::treeview pathName ?options?]
dgw::mixin $tv dgw::tvband ?options?
\f[R]
.fi
.SS DESCRIPTION
.PP
The package \f[B]dgw::tvmixins\f[R] implements several
\f[I]snit::widgetadaptor\f[R]s which extend the standard
\f[I]ttk::treeview\f[R] widget with different functionalities.
Different adaptors can be chained together to add the required
functionalities.
Furthermore at any later time point using the \f[I]dgw::mixin\f[R]
command other adaptors can be installed on the widget.
.SS WIDGET COMMANDS
.PP
\f[B]dgw::mixin\f[R] \f[I]pathName mixinWidget ?\-option value
\&...?\f[R]
.PP
Adds the properties and methods of a snit::widgetadaptor specified with
\f[I]mixinWidget\f[R] to the exising widget created before with the
given \f[I]pathName\f[R] and configures the widget using the given
\f[I]options\f[R].
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: mixin
# standard treeview widget
set tv [ttk::treeview .tv \-columns \[dq]A B C\[dq] \-show headings]
$tv heading A \-text A
$tv heading B \-text B
$tv heading C \-text C
pack $tv \-side top \-fill both \-expand true
# add  sorting after object creation using the mixin command
dgw::mixin $tv dgw::tvsortable
# fill the widget
for {set i 0} {$i < 20} {incr i} { 
$tv insert {} end \-values [list [expr {rand()*4}] \[rs]
[expr {rand()*10}] [expr {rand()*20}]] 
}
# add another widget adaptor
dgw::mixin $tv dgw::tvband
# configure the new options of this adaptor at a later point
$tv configure  \-bandcolors [list white ivory]
\f[R]
.fi
.RE
.PP
\f[B]dgw::tvband\f[R] \f[I][ttk::treeview pathName] ?\-option value
\&...?\f[R]
.PP
Creates and configures the \f[I]dgw::tvband\f[R] widget using the Tk
window id \f[I]pathName\f[R] and the given \f[I]options\f[R].
Please note that this adaptor might have performace issues and that the
\f[I]ttk::treeview\f[R] widget of Tk 8.7 probably will have a configure
option \f[I]\-striped\f[R] and \f[I]\-stripedbackgroundcolor\f[R] which
can replace this adaptor.
.PP
The following option is available:
.RS
.IP \[bu] 2
\f[I]\-bandcolors\f[R] \f[I]list\f[R] \- list of the two colors to be
displayed alternatively.
.RE
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: tvband
dgw::tvband [ttk::treeview .fb \-columns [list A B C] \-show headings]
foreach col [list A B C] { .fb heading $col \-text $col }
for {set i 0} {$i < 20} {incr i 1} {
\&.fb insert {} end \-values [list  [expr {int(rand()*100)}] \[rs]
[expr {int(rand()*1000)}] [expr {int(rand()*1000)}]]
}
pack .fb \-side top \-fill both \-expand yes
\f[R]
.fi
.RE
.PP
\f[B]dgw::tvedit\f[R] \f[I][ttk::treeview pathName] ?\-option value
\&...?\f[R]
.PP
Creates and configures the \f[I]dgw::tvedit\f[R] widget using the Tk
window id \f[I]pathName\f[R] and the given \f[I]options\f[R].
This widget adaptor allows to do in place edits of the text within the
ttk::treeview widget.
The code is largly based on the wiki code in Inplace edit in
ttk::treeview (https://wiki.tcl-lang.org/page/Inplace+edit+in+ttk%3A%3Atreeview).
Note: Currently only tabular, non hierarchical \f[I]ttk::treeview\f[R]
widget\[cq]s can be edited.
.PP
The following options are available:
.RS
.IP \[bu] 2
\f[I]\-edittypes\f[R] \f[I]list\f[R] \- list of key values pairs where
the key is the colummn name and the values are pssible data types or
lists of available values.
The following data types are available
.IP "1." 3
\f[I]bool\f[R] provides a boolean value selection of true and false
using a check box
.IP "2." 3
\f[I]int\f[R] a integer range of values must be given as: \f[I]int [list
start end]\f[R]
.IP "3." 3
\f[I]list\f[R] list of possible values must be given with the values
such as: \f[I]list [list A B C D E]\f[R]
.IP "4." 3
the default if no type is provided for a column name is a text entry
with free text edition available
.RE
.RS
.IP \[bu] 2
\f[I]\-editdefault\f[R] \f[I]type\f[R] the default edit is a entry text
field, if you set this to an empty string only columns listed in the
\f[I]\-edittypes\f[R] options can be edited.
.RE
.RS
.IP \[bu] 2
\f[I]\-editendcmd\f[R] \f[I]commandName\f[R] the command to be executed
after the value was changed.
The widget path, the data type, the row id, the old and the new value
are added as command arguments.
This method can be used to validate the input as well and to perform
some actions after the entry was edited.
.RE
.PP
The widget provides the follwing events:
.RS
.IP \[bu] 2
<<\f[I]TreeviewEditEnd\f[R]>> which is fired if a entry in the
\f[I]ttk::treeview\f[R] widget is changed.
The following event symbols are available: \f[I]%d\f[R] a list of the
row index and the column name which was changed, \f[I]%W\f[R] (widget).
.RE
.PP
Bindings:
.RS
.IP \[bu] 2
\f[I]\f[R] \- edit current row entries
.IP \[bu] 2
\f[I]\f[R] \- cancel edits
.IP \[bu] 2
\f[I]\f[R] \- save edit and end current edits
.IP \[bu] 2
\f[I]\f[R] \- switch to the next edit field
.IP \[bu] 2
\f[I]\f[R] \- switch to the previous edit field
.RE
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: tvedit
proc editDone {args} {
puts \[dq]done: $args\[dq]
}
pack [dgw::tvedit [ttk::treeview .tv \-columns {bool int list} \-show {headings} \[rs]
\-selectmode extended \-yscrollcommand {.sb set}] \[rs]
\-edittypes [list bool bool int [list int 0 100]] \[rs]
\-editdefault \[dq]\[dq] \-editendcmd editDone] \-fill both \-expand true \-side left
pack [ttk::scrollbar .sb \-orient v \-command \[dq].tv yview\[dq]] \-fill y \-side left
\&.tv insert {} end \-values {true 15 {Letter B}}
\&.tv insert {} end \-values {true 35 {Letter D}}
for {set i 0} {$i<20} {incr i} {
\&.tv insert {} end \-values [list true $i {Letter B}]
}
dgw::mixin .tv dgw::tvband
\f[R]
.fi
.RE
.PP
\f[B]dgw::tvfilebrowser\f[R] \f[I][ttk::treeview pathName] ?\-option
value \&...?\f[R]
.PP
Creates and configures the \f[I]dgw::tvfilebrowser\f[R] widget using the
Tk window id \f[I]pathName\f[R] and the given \f[I]options\f[R].
.PP
The following option is available:
.RS
.IP \[bu] 2
\f[I]\-directory dirName\f[R] \- starting directory for the filebrowser,
default current directory.
.IP \[bu] 2
\f[I]\-browsecmd cmdName\f[R] \- command to be executed if the users
double clicks on a row item or presses the Return key.
The widgets \f[I]pathName\f[R] and the actual row index are appended to
the \f[I]cmdName\f[R] as arguments, default to empty string.
.IP \[bu] 2
\f[I]\-fileimage imgName\f[R] \- image to be displayed as filename image
left of the filename, default is standard file icon.
.RE
.PP
The following method(s) is(are) available:
.RS
.IP \[bu] 2
\f[I]browseDir dirName\f[R] \- the directory to be loaded into the
\f[I]dgw::tvfilebrowser\f[R] widget.
.RE
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: tvfilebrowser
dgw::tvfilebrowser [dgw::tvsortable [dgw::tvksearch [dgw::tvband \[rs]
[ttk::treeview .fb]]] \[rs]
\-sorttypes [list Name directory Size real Modified dictionary]]
pack .fb \-side top \-fill both \-expand yes
\f[R]
.fi
.RE
.PP
\f[B]dgw::tvksearch\f[R] \f[I][ttk::treeview pathName] ?\-option value
\&...?\f[R]
.PP
Creates and configures the \f[I]dgw::tvksearch\f[R] widget using the Tk
window id \f[I]pathName\f[R] and the given \f[I]options\f[R].
With this widget you can use the Home and End keys for navigation and
further letter typing starts searching in the first column shifting
focus and display to the current matching entry.
.PP
There are currently no options or methods available for this widget.
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: tvksearch
dgw::tvfilebrowser [dgw::tvksearch [ttk::treeview .fb]]
pack .fb \-side top \-fill both \-expand yes
\f[R]
.fi
.RE
.PP
\f[B]dgw::tvsortable\f[R] \f[I][ttk::treeview pathName] ?\-option value
\&...?\f[R]
.PP
Creates and configures the \f[I]dgw::tvsortable\f[R] widget using the Tk
window id \f[I]pathName\f[R] and the given \f[I]options\f[R].
.PP
The following option is available:
.RS
.IP \[bu] 2
\f[I]\-sorttypes\f[R] the options for the \f[I]lsort\f[R] command for
each of the columns, such as dictionary, ascii, real etc.
Default: real.
The values are given as a list of key\-value pairs where the key is the
column name.
In addition to teh standard \f[I]lsort\f[R] options as well the option
directory can be given if the widget contains results of a directory
listening with filenames and directory names.
In this case the directories are always sorted above the filenames.
.RE
.PP
The following methods are available:
.RS
.IP \[bu] 2
\f[I]sortBy\f[R] \f[I]colId decreasing\f[R] \- sort widget by column
with the given \f[I]colId\f[R] and in decreasing order if true or
\f[I]increasing\f[R] if false.
.IP \[bu] 2
\f[I]reSort\f[R] redo the last sorting again, useful if the data in teh
widget where changed either interactively for instance using the tvedit
adaptor or programmatically.
.RE
.PP
The widget further provides the following event:
.IP \[bu] 2
<<\f[I]SortEnd\f[R]>> \- with the following symbols \f[I]%W\f[R] (widget
path) and \f[I]%d\f[R] (column id)
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: tvsortable
dgw::tvsortable [dgw::tvband [ttk::treeview .fb \-columns [list A B C] \[rs]
\-show headings]] \-sorttypes [list A real B real C integer]
foreach col [list A B C] { .fb heading $col \-text $col }
for {set i 0} {$i < 20} {incr i 1} {
\&.fb insert {} end \-values [list  [expr {int(rand()*100)}] \[rs]
[expr {int(rand()*1000)}] [expr {int(rand()*1000)}]]
}
pack .fb \-side top \-fill both \-expand yes
\f[R]
.fi
.RE
.PP
\f[B]dgw::tvtooltip\f[R] \f[I][ttk::treeview pathName] ?\-option value
\&...?\f[R]
.PP
Creates and configures the \f[I]dgw::tvtooltip\f[R] widget using the Tk
window id \f[I]pathName\f[R] and the given \f[I]options\f[R].
.PP
There are currently no options available.
.PP
The widget provides the following events:
.IP \[bu] 2
<<RowEnter>> with the following symbols: %d the row index, and the
standards %W (widget), %x (widgetX), %y (widgetY), %X (rootx), %Y
(rootY)
.IP \[bu] 2
<<RowLeave>> with the following symbols: %d the row index, and the
standards %W (widget), %x (widgetX), %y (widgetY), %X (rootx), %Y
(rootY)
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: tvtooltip
set fb [dgw::tvtooltip [dgw::tvfilebrowser [ttk::treeview .fp2] \[rs]
\-directory . \-fileimage movie \[rs]
\-filepattern {\[rs].(3gp|mp4|avi|mkv|mp3|ogg)$}]]
pack $fb \-side top \-fill both \-expand yes
pack [::ttk::label .msg \-font \[dq]Times 12 bold\[dq] \-textvariable ::msg \-width 20 \[rs]
\-background salmon \-borderwidth 2 \-relief ridge] \[rs]
\-side top \-fill x \-expand false \-ipadx 5 \-ipady 4
bind $fb <<RowEnter>> { set ::msg \[dq]  Entering row %d\[dq]}
bind $fb <<RowLeave>> { set ::msg \[dq]  Leaving row %d\[dq]}
\f[R]
.fi
.RE
.PP
\f[B]dgw::tvtree\f[R] \f[I][ttk::treeview pathName] ?\-option value
\&...?\f[R]
.PP
Creates and configures the \f[I]dgw::tvtree\f[R] widget using the Tk
window id \f[I]pathName\f[R] and the given \f[I]options\f[R].
.PP
There is(are) currently the following option(s) available:
.IP \[bu] 2
\f[I]\-icon\f[R] \- the icon type, which can be currently either book or
folder.
To provide your own icons you must create two image icons open16 and
close16.
Support for icons of size 22 will be added later.
.PP
The widget provides the following event:
.IP \[bu] 2
<<InsertItem>> which is fired if a item is inserted into the
\f[I]tvtree\f[R] widget, there are the following event symbols
available: \f[I]%d\f[R] the row index, and the standard \f[I]%W\f[R]
(widget pathname).
.PP
Example:
.RS
.IP
.nf
\f[C]
# demo: tvtree
set tree [dgw::tvtree [ttk::treeview .tree \[rs]
\-height 15 \-show tree \-selectmode browse] \[rs]
\-icon folder]
foreach txt {first second third} {
set id [$tree insert {} end \-text \[dq] $txt item\[dq] \-open 1]
for {set i [expr {1+int(rand()*5)}]} {$i > 0} {incr i \-1} {
set child [$tree insert $id 0 \-text \[dq] child $i\[dq]]
for {set j [expr {int(rand()*3)}]} {$j > 0} {incr j \-1} {
$tree insert $child 0 \-text \[dq] grandchild $i\[dq]
}
}
}
pack $tree \-side top \-fill both \-expand true
\f[R]
.fi
.RE
.SS EXAMPLE
.PP
In the examples below we create first a filebrowser widget using all the
widget adaptors for demonstration purposes.
Therafter comes a tooltip demonstration, a tree demonstration and
finally a demonstration on how to use the \f[I]dgw::mixin\f[R] command
which simplifies the addition of new behaviors to our
\f[I]ttk::treewidget\f[R].
.IP
.nf
\f[C]
# wrapper function 
proc fbrowse {path args} {
     set fb [dgw::tvtooltip [dgw::tvsortable [dgw::tvksearch \[rs]
        [dgw::tvfilebrowser [dgw::tvband \[rs]
        [ttk::treeview $path]] {*}$args]] \[rs]
         \-sorttypes [list Name directory Size real Modified dictionary]]]
     return $fb
}
set pw [ttk::panedwindow .pw \-orient horizontal]
set f0 [ttk::frame $pw.f]
set f1 [ttk::frame $f0.f]
set fb [fbrowse $f1.fb]
pack $fb \-side left \-fill both \-expand yes
pack [ttk::scrollbar $f1.yscroll \-command [list $fb yview]] \[rs]
      \-side left \-fill y \-expand false
$fb configure \-yscrollcommand [list $f1.yscroll set]
pack $f1 \-side top \-fill both \-expand true
# demo tvtooltip
pack [::ttk::label $f0.msg \-font \[dq]Times 12 bold\[dq] \-textvariable ::msg \-width 20 \[rs]
     \-background salmon \-borderwidth 2 \-relief ridge] \[rs]
     \-side top \-fill x \-expand false \-ipadx 5 \-ipady 4
bind $fb <<RowEnter>> { set ::msg \[dq]  Entering row %d\[dq]}
bind $fb <<RowLeave>> { set ::msg \[dq]  Leaving row %d\[dq]}

$pw add $f0
set tree [dgw::tvtree [ttk::treeview $pw.tree \-height 15 \-show tree \-selectmode browse] \-icon folder]
foreach txt {first second third} {
   set id [$tree insert {} end \-text \[dq] $txt item\[dq] \-open 1]
   for {set i [expr {1+int(rand()*5)}]} {$i > 0} {incr i \-1} {
       set child [$tree insert $id 0 \-text \[dq] child $i\[dq]]
       for {set j [expr {int(rand()*3)}]} {$j > 0} {incr j \-1} {
          $tree insert $child 0 \-text \[dq] grandchild $i\[dq]
       }
   }
}
$pw add $tree
# another example using mixin syntax
set tv [ttk::treeview $pw.tv \-columns \[dq]A B C\[dq] \-show headings]
dgw::mixin $tv dgw::tvsortable

$tv heading A \-text A
$tv heading B \-text B
$tv heading C \-text C
$pw add $tv
for {set i 0} {$i < 20} {incr i} { 
    $tv insert {} end \-values [list [expr {rand()*4}] \[rs]
        [expr {rand()*10}] [expr {rand()*20}]] 
}
dgw::mixin $tv dgw::tvband
$tv configure  \-bandcolors [list white ivory]
pack $pw \-side top \-fill both \-expand true
\f[R]
.fi
.SS INSTALLATION
.PP
Installation is easy you can install and use the \f[B]dgw::tvmixins\f[R]
package if you have a working install of:
.IP \[bu] 2
the snit package which can be found in tcllib \-
https://core.tcl\-lang.org/tcllib (https://core.tcl-lang.org/tcllib)
.PP
For installation you copy the complete \f[I]dgw\f[R] folder into a path
of your \f[I]auto_path\f[R] list of Tcl or you append the
\f[I]auto_path\f[R] list with the parent dir of the \f[I]dgw\f[R]
directory.
Alternatively you can install the package as a Tcl module by creating a
file dgw/tvmixins\-0.3.tm in your Tcl module path.
.PP
Only if you you like to extract the HTML documentation and run the
examples, you need the complete dgw package and for the HTML generation
the tcllib Markdown package.
.SS DEMO
.PP
Example code for this package can be executed by running this file using
the following command line:
.IP
.nf
\f[C]
$ wish tvmixins.tcl \-\-demo
\f[R]
.fi
.PP
Specific code examples outside of the EXAMPLE section can be executed
using the string after the \f[C]demo:\f[R] string such as:
.IP
.nf
\f[C]
$ wish tvmixins.tcl \-\-demo tvband
\f[R]
.fi
.PP
The example code used for the demo in the EXAMPLE section can be seen in
the terminal by using the following command line:
.IP
.nf
\f[C]
$ tclsh tvmixins.tcl \-\-code
\f[R]
.fi
.SS DOCUMENTATION
.PP
The script contains embedded the documentation in Markdown format.
To extract the documentation you should use the following command line:
.IP
.nf
\f[C]
$ tclsh tvmixins.tcl \-\-markdown
\f[R]
.fi
.PP
This will extract the embedded manual pages in standard Markdown format.
You can as well use this markdown output directly to create html pages
for the documentation by using the \f[I]\[en]html\f[R] flag.
.IP
.nf
\f[C]
$ tclsh tvmixins.tcl \-\-html
\f[R]
.fi
.PP
If the tcllib Markdown package is installed, this will directly create a
HTML page \f[C]tvmixins.html\f[R] which contains the formatted
documentation.
.PP
Github\-Markdown can be extracted by using the \f[I]\[en]man\f[R]
switch:
.IP
.nf
\f[C]
$ tclsh tvmixins.tcl \-\-man
\f[R]
.fi
.PP
The output of this command can be used to feed a Markdown processor for
conversion into a html, docx or pdf document.
If you have pandoc installed for instance, you could execute the
following commands:
.IP
.nf
\f[C]
tclsh ../tvmixins.tcl \-\-man > tvmixins.md
pandoc \-i tvmixins.md \-s \-o tvmixins.html
pandoc \-i tvmixins.md \-s \-o tvmixins.tex
pdflatex tvmixins.tex
\f[R]
.fi
.SS SEE ALSO
.IP \[bu] 2
dgw package
homepage (https://chiselapp.com/user/dgroth/repository/tclcode/index) \-
various useful widgets
.IP \[bu] 2
ttk::treeview widget
manual (https://www.tcl.tk/man/tcl8.6/TkCmd/ttk_treeview.htm) standard
manual page for the ttk::treeview widget
.SS CHANGES
.IP \[bu] 2
2020\-04\-10 \- version 0.2 released with adaptors: \f[I]tvband\f[R],
\f[I]tvfilebrowser\f[R], \f[I]tvksearch\f[R], \f[I]tvsortable\f[R],
\f[I]tvtooltip\f[R]
.IP \[bu] 2
2020\-04\-14 \- version 0.3 released with adaptor \f[I]tvtree\f[R],
\f[I]tvedit\[cq] and command \f[R]dgw::mixin*
.SS TODO
.IP \[bu] 2
tests
.IP \[bu] 2
github url
.SS AUTHORS
.PP
The \f[B]dgw::tvmixins\f[R] widget was written by Detlef Groth,
Schwielowsee, Germany.
.SS Copyright
.PP
Copyright (c) 2020 Dr.\ Detlef Groth, E\-mail: detlef(at)dgroth(dot)de
.SS LICENSE
.PP
dgw::tvmixins package, version 0.3.
.PP
Copyright (c) 2019\-2020 Dr.\ Detlef Groth, E\-mail:
detlef(at)dgroth(dot)de
.PP
This library is free software; you can use, modify, and redistribute it
for any purpose, provided that existing copyright notices are retained
in all copies and that this notice is included verbatim in any
distributions.
.PP
This software is distributed WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
.SH AUTHORS
Detlef Groth, Schwielowsee, Germany.
